use serde_json::Value::Bool;
use serde_json::Value;

use std::io::Write;

fn execute_php(command: &str, arguments: &[&str]) -> String {
    let mut execute_array: Vec<&str> = Vec::new();
    execute_array.extend(&["-r", &command, "--"]);
    execute_array.extend(arguments);
    let output = std::process::Command::new("php")
        .args(execute_array)
        .output()
        .expect("");
    let mut buff_stderr = std::io::BufWriter::new(std::io::stderr());
    buff_stderr
        .write(&output.stderr)
        .expect("Unable to write to stderr.");
    buff_stderr.flush().expect("Unable to flush stderr.");
    return String::from_utf8(output.stdout).expect("Unable to parse php response");
}

pub fn ls(path: &str, user: &str, nextcloud_location: &str) -> Vec<String> {
    let php_script: String = [
        &start_script(),
        "$path = $argv[3];",
        "function get_name($node) { return $node->getName(); }",
        "echo(json_encode(array_map('get_name', $view->getDirectoryContent($path))));",
    ]
    .join("\n");
    let result = execute_php(&php_script, &[user, nextcloud_location, path]);
    let result: Vec<String> = serde_json::from_str(&result).expect("Invalid json reading ls.");
    result
}

pub fn check_if_exists(path: &str, user: &str, nextcloud_location: &str) -> bool {
    let php_script: String = [
        &start_script(),
        "$path = $argv[3];",
        "echo json_encode($view->file_exists($path));",
    ]
    .join("\n");
    json_get_boolean(&execute_php(&php_script, &[user, nextcloud_location, path]))
}

pub fn check_if_is_dir(path: &str, user: &str, nextcloud_location: &str) -> bool {
    let php_script: String = [
        &start_script(), 
        "$path = $argv[3];",
        "echo json_encode($view->is_dir($path));",
    ]
    .join("\n");
    json_get_boolean(&execute_php(&php_script, &[user, nextcloud_location, path]))
}
fn json_get_boolean(json: &str) -> bool {
    let json_value: Value = serde_json::from_str(json).expect("Unable to parse json at json_get_boolean.");
    match json_value {
        Bool(x) => x,
        _ => false,
    }
}

pub fn start_script() -> String {
    [
        "define('OC_CONSOLE', 1);",
        "$user = $argv[1];",
        "$nextcloud_location = $argv[2];",
        "include $nextcloud_location . '/lib/base.php';",
        "use Symfony\\Component\\Console\\Application as SymfonyApplication;",
        "use \\OC\\Files\\View;",
        "$application = new SymfonyApplication(
            \\OC::$server->getThemingDefaults()->getName(), 
            \\OC_Util::getVersionString()
        );",
        "\\OC_Util::setupFS($user);",
        "$view = new View('/'. $user . '/files');",
    ]
    .join("\n")
}
